# Create your views here.
# import the logging library
import logging
import os
import traceback
from typing import Dict

from fr_emac_gind_rio.models.generated.interpretation_model_api import pjaxb_learnASync, \
    pjaxb_learnASyncResponse, pjaxb_predict, pjaxb_predictResponse, pjaxb_applyPredefinedLabelMethod, \
    pjaxb_applyPredefinedLabelMethodResponse
from fr_emac_gind_rio.interpretation_model.interpretation_marshaller import InterpretationMarshaller
from fr_emac_gind_rio.machine_learning_engine.learning_module_manager import LearningModuleManager
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

# Get an instance of a logger

logger = logging.getLogger(__name__)


@api_view(['GET', 'POST'])
def learnASync(request):

    """
    learn
    """
    if request.method == 'GET':
        try:
            logger.info("learn get")
            return Response(LearningModuleManager().plugin_list(), status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    elif request.method == 'POST':
        try:
            logger.info("learn post")
            os.write(1, b'    learn  \n')
            print(request.data)
            os.write(1, str.encode(str(request.data) + "\n"))
            learn_async: pjaxb_learnASync = InterpretationMarshaller().marshall(request.data, pjaxb_learnASync)

            response: pjaxb_learnASyncResponse = LearningModuleManager().learn_async(learn_async)

            return Response(InterpretationMarshaller().unmarshall(response), status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET', 'POST'])
def predict(request):

    """
    predict
    """
    if request.method == 'GET':
        try:
            logger.info("predict get")
            return Response(LearningModuleManager().plugin_list(), status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    elif request.method == 'POST':
        try:
            logger.info("predict post")
            os.write(1, b'    predict  \n')
            print(request.data)
            #os.write(1, str.encode(str(request.data) + "\n"))
            pred: pjaxb_predict = InterpretationMarshaller().marshall(request.data, pjaxb_predict)
            response: pjaxb_predictResponse = LearningModuleManager().predict(pred)

            json_resp: Dict = InterpretationMarshaller().unmarshall(response)
            return Response(json_resp, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            os.write(1, bytes(str(e), 'utf-8'))
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET', 'POST'])
def predefined_labels_methods(request):

    """
    labels
    """
    if request.method == 'GET':
        try:
            logger.info("predefined_labels_methods get")
            return Response(LearningModuleManager().predefined_labels_methods_list(), status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
    elif request.method == 'POST':
        try:
            logger.info("predefined_labels_methods post")
            os.write(1, b'    predict  \n')
            pred: pjaxb_applyPredefinedLabelMethod = InterpretationMarshaller().marshall(request.data, pjaxb_applyPredefinedLabelMethod)
            response: pjaxb_applyPredefinedLabelMethodResponse = LearningModuleManager().apply_predefined_label_method(pred)

            json_resp: Dict = InterpretationMarshaller().unmarshall(response)
            return Response(json_resp, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            os.write(1, bytes(str(e), 'utf-8'))
            return Response({ "error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)




