import threading
import traceback
from typing import Dict, Optional

from fr_emac_gind_rio.interpretation.abstract_learning_plugin import AbstractLearningPlugin
from fr_emac_gind_rio.models.generated.interpretation_model_api import pjaxb_learnASync, pjaxb_learningResult, \
    pjaxb_lastLearningResult
from fr_emac_gind_rio.interpretation.soap_callback_learning_client import SoapCallBackASyncClient


class LearnASyncThread(threading.Thread):

    context: Dict
    client: SoapCallBackASyncClient = None
    requestId: Optional[str] = None
    collaborationName: Optional[str] = None
    knowledgeSpaceName: Optional[str] = None

    def __init__(self, module: AbstractLearningPlugin, learn_async: pjaxb_learnASync, context: Dict):
        threading.Thread.__init__(self)
        self.name = module.get_name() + "_" + learn_async.requestId
        self.module = module
        self.learn_async = learn_async
        self.context = context
        self.requestId = learn_async.requestId
        self.collaborationName = learn_async.collaborationName
        self.knowledgeSpaceName = learn_async.knowledgeSpaceName
        self.client = SoapCallBackASyncClient(context=self.context, requestId=self.requestId,
                                              collaborationName=self.collaborationName,
                                              knowledgeSpaceName=self.knowledgeSpaceName)

    def run(self):
        print("Starting " + self.name)
        try:
            self.module.learn_async(learn_async_req=self.learn_async, context=self.context)
        except Exception as e:
            print("Error " + self.name)
            print(e)
            traceback.print_exc()
            result: pjaxb_lastLearningResult = self.context.get("lastLearningResult")
            if e.__dict__.__contains__("message"):
                result.commonLearningResult.commonReportMsg = "Error on " + self.name + ": " + e.__dict__["message"]
            elif e.__dict__.__contains__("_message"):
                result.commonLearningResult.commonReportMsg = "Error on " + self.name + ": " + e.__dict__["_message"]
            else:
                result.commonLearningResult.commonReportMsg = "Error on " + self.name + ": " + str(e.args[0])
            self.client.send_learning_result(result)
        print("Exiting " + self.name)
