# Create your views here.
# import the logging library
import json
import logging
import os
import sys
import traceback
from typing import Optional, List, Dict

from fr_emac_gind_rio.interpretation.abstract_learning_plugin import AbstractLearningPlugin
from fr_emac_gind_rio.models.generated.interpretation_model_api import pjaxb_deployResponse, \
    pjaxb_interpretationConfig, pjaxb_undeployResponse, pjaxb_deployResult, pjaxb_context
from fr_emac_gind_rio.interpretation_model.interpretation_marshaller import InterpretationMarshaller
from fr_emac_gind_rio.machine_learning_engine.learning_module_manager import LearningModuleManager
from fr_emac_gind_rio.utils.qname_helper import QNameHelper
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response

# Get an instance of a logger

logger = logging.getLogger(__name__)


@api_view(['GET'])
def info(request):
    """
    List all infos
    """
    if request.method == 'GET':
        try:
            logger.info("info get")
            info_rio_sys: Dict = {
                "python-version": sys.version,
            }
            print(os.environ['rio_config_ml_engine'])
            info_rio_sys.update(json.loads(os.environ['rio_config_ml_engine'].replace("'", "\"")))
            return Response(info_rio_sys, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            os.write(1, str.encode(str(e) + "\n"))
            return Response({"error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET', 'POST'])
def deploy(request):
    """
    Deploy: List all interpretation_config deployed by collaboration and knowledge
    """
    if request.method == 'GET':
        try:
            logger.info("ml_manager get")
            collaboration_name: Optional[str] = request.query_params.get("collaborationName")
            knowledge_space_name: Optional[str] = request.query_params.get("knowledgeSpaceName")
            if collaboration_name is None or knowledge_space_name is None:
                return Response(LearningModuleManager().plugin_list(), status=status.HTTP_200_OK)
            else:
                deployed_modules: List[str] = []
                if LearningModuleManager().deployed_configs_by_ckn.get(collaboration_name) is not None and \
                        LearningModuleManager().deployed_configs_by_ckn[collaboration_name].get(
                            knowledge_space_name) is not None:
                    deployed_modules = LearningModuleManager().deployed_configs_by_ckn[collaboration_name][
                        knowledge_space_name].keys()
                return Response(deployed_modules, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            os.write(1, str.encode(str(e) + "\n"))
            return Response({"error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    elif request.method == 'POST':
        try:
            logger.info("deploy interpretation config")
            os.write(1, b'    deploy interpretation config  \n')
            os.write(1, str.encode(str(request.data) + "\n"))
            print(request.data)

            inter_config_json: Dict = request.data['interpretationConfig']
            inter_config: pjaxb_interpretationConfig = InterpretationMarshaller().marshall(inter_config_json,
                                                                                           pjaxb_interpretationConfig)

            ctxt: pjaxb_context = inter_config.context
            collaboration_name: str = ctxt.collaborationName
            knowledge_space_name: str = ctxt.knowledgeSpaceName
            plugin_name: str = inter_config.name

            context: Dict = LearningModuleManager().create_default_context()
            if inter_config is not None and inter_config.context is not None and inter_config.context.resourcesFolderDir is not None:
                context["resources_folder_base_dir"] = inter_config.context.resourcesFolderDir

            plugin: AbstractLearningPlugin = LearningModuleManager().get_or_deploy_with_default_config(collaboration_name=collaboration_name,
                                                                      knowledge_space_name=knowledge_space_name,
                                                                      plugin_name=plugin_name,
                                                                      context=context,
                                                                      config=inter_config)

            response: pjaxb_deployResponse = pjaxb_deployResponse()
            response.result = pjaxb_deployResult()
            response.result.add_name(plugin_name)

            return Response(InterpretationMarshaller().unmarshall(response), status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
           # os.write(1, str.encode(str(e) + "\n"))
            return Response({"error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET', 'POST'])
def undeploy(request):
    """
    Undeploy: List all interpretation_config deployed by collaboration and knowledge
    """
    if request.method == 'GET':
        try:
            logger.info("ml_manager get")
            collaboration_name: Optional[str] = request.query_params.get("collaborationName")
            knowledge_space_name: Optional[str] = request.query_params.get("knowledgeSpaceName")
            if collaboration_name is None or knowledge_space_name is None:
                return Response(LearningModuleManager().list(), status=status.HTTP_200_OK)
            else:
                deployed_modules: List[str] = []
                if LearningModuleManager().deployed_configs_by_ckn.get(collaboration_name) is not None and \
                        LearningModuleManager().deployed_configs_by_ckn[collaboration_name].get(
                            knowledge_space_name) is not None:
                    deployed_modules = LearningModuleManager().deployed_configs_by_ckn[collaboration_name][
                        knowledge_space_name].keys()
                return Response(deployed_modules, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            os.write(1, str.encode(str(e) + "\n"))
            return Response({"error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    elif request.method == 'POST':
        try:
            logger.info("undeploy interpretation config")
            os.write(1, b'    undeploy interpretation config  \n')
            os.write(1, str.encode(str(request.data) + "\n"))
            print(request.data)

            response: pjaxb_undeployResponse = LearningModuleManager().undeploy(
                collaboration_name=request.data['collaborationName'],
                knowledge_space_name=request.data['knowledgeSpaceName'],
                plugin_name=QNameHelper.get_local_part(request.data['pluginName']))

            return Response({'res': 'true'}, status=status.HTTP_200_OK)
        except Exception as e:
            print(e)
            traceback.print_exc()
            return Response({"error_msg": str(e.args[0])}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
