#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import json
import logging
import logging.config
import os
import sys
from typing import Dict

from fr_emac_gind_rio.utils.utils import Utils


def main():
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'fr_emac_gind_rio.machine_learning_engine.settings')
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    print("start machine learning engine")
    config_file: str = None
    resources_folder_base_dir: str = None
    for arg in sys.argv:
        if "-config" in arg:
            config_file = arg[len("-config="): len(arg)]
        if "-resources_folder_base_dir" in arg:
            resources_folder_base_dir = arg[len("-resources_folder_base_dir="): len(arg)]
            print("resources folder: " + resources_folder_base_dir)




    # create logger
    logger = logging.getLogger(__name__)

    logger.info("Start Machine Learning Engine with TensorFlow")

    config: Dict = {}

    if config_file is not None:
        sys.argv.remove("-config=" + config_file)
        sys.argv.append("runserver")
        config.update(Utils().get_config(config_file))
        sys.argv.append("0.0.0.0:" + str(config['python-ml-engine-port']))

    if resources_folder_base_dir is not None:
        sys.argv.remove("-resources_folder_base_dir=" + resources_folder_base_dir)
    else:
        resources_folder_base_dir = os.path.realpath(os.path.dirname(os.path.realpath(__file__)) + "../../../../java/packaging/gind-r-io-standalone/target/resourcesFolder")
    config["resources_folder_base_dir"] = resources_folder_base_dir

    if not os.path.exists(resources_folder_base_dir):
        os.makedirs(resources_folder_base_dir)

    if os.environ.get('rio_config_ml_engine') is None:
        os.environ['rio_config_ml_engine'] = json.dumps(config)

    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()
